@@ -6,7 +6,7 @@
 #endif
 
 #ifdef USE_DEFAULT_SIGNATURES
-{-# LANGUAGE DefaultSignatures, TypeFamilies #-}
+{-# LANGUAGE DefaultSignatures, TypeFamilies, TypeOperators #-}
 #endif
 
 #if !MIN_VERSION_base(4,6,0)
@@ -61,7 +61,7 @@ module Text.Parser.Combinators
   ) where
 
 import Control.Applicative
-import Control.Monad (MonadPlus(..))
+import Control.Monad (MonadPlus(..), void)
 import Control.Monad.Trans.Class
 import Control.Monad.Trans.State.Lazy as Lazy
 import Control.Monad.Trans.State.Strict as Strict
@@ -71,8 +71,9 @@ import Control.Monad.Trans.RWS.Lazy as Lazy
 import Control.Monad.Trans.RWS.Strict as Strict
 import Control.Monad.Trans.Reader
 import Control.Monad.Trans.Identity
-import Data.Foldable (asum)
-import Data.List.NonEmpty
+import qualified Data.Foldable as F
+import qualified Data.List.NonEmpty as NonEmpty
+import Data.List.NonEmpty (NonEmpty(..))
 #if __GLASGOW_HASKELL__ < 710
 import Data.Monoid
 #ifdef ORPHAN_ALTERNATIVE_READP
@@ -80,9 +81,16 @@ import Data.Orphans ()
 #endif
 import Data.Traversable (sequenceA)
 #endif
+
+#ifdef MIN_VERSION_parsec
 import qualified Text.Parsec as Parsec
+#endif
+
+#ifdef MIN_VERSION_attoparsec
 import qualified Data.Attoparsec.Types as Att
 import qualified Data.Attoparsec.Combinator as Att
+#endif
+
 import qualified Text.ParserCombinators.ReadP as ReadP
 
 #ifdef MIN_VERSION_binary
@@ -90,11 +98,15 @@ import Control.Monad (when, unless)
 import qualified Data.Binary.Get as B
 #endif
 
+#if MIN_VERSION_base(4,9,0)
+import Control.Monad (replicateM)
+#endif
+
 -- | @choice ps@ tries to apply the parsers in the list @ps@ in order,
 -- until one of them succeeds. Returns the value of the succeeding
 -- parser.
 choice :: Alternative m => [m a] -> m a
-choice = asum
+choice = F.asum
 {-# INLINE choice #-}
 
 -- | @option x p@ tries to apply parser @p@. If @p@ fails without
@@ -110,7 +122,7 @@ option x p = p <|> pure x
 -- It only fails if @p@ fails after consuming input. It discards the result
 -- of @p@. (Plays the role of parsec's optional, which conflicts with Applicative's optional)
 skipOptional :: Alternative m => m a -> m ()
-skipOptional p = (() <$ p) <|> pure ()
+skipOptional p = void p <|> pure ()
 {-# INLINE skipOptional #-}
 
 -- | @between open close p@ parses @open@, followed by @p@ and @close@.
@@ -138,7 +150,7 @@ sepBy p sep = sepBy1 p sep <|> pure []
 -- | @sepBy1 p sep@ parses /one/ or more occurrences of @p@, separated
 -- by @sep@. Returns a list of values returned by @p@.
 sepBy1 :: Alternative m => m a -> m sep -> m [a]
-sepBy1 p sep = toList <$> sepByNonEmpty p sep
+sepBy1 p sep = F.toList <$> sepByNonEmpty p sep
 {-# INLINE sepBy1 #-}
 
 -- | @sepByNonEmpty p sep@ parses /one/ or more occurrences of @p@, separated
@@ -151,7 +163,7 @@ sepByNonEmpty p sep = (:|) <$> p <*> many (sep *> p)
 -- separated and optionally ended by @sep@. Returns a list of values
 -- returned by @p@.
 sepEndBy1 :: Alternative m => m a -> m sep -> m [a]
-sepEndBy1 p sep = toList <$> sepEndByNonEmpty p sep
+sepEndBy1 p sep = F.toList <$> sepEndByNonEmpty p sep
 
 -- | @sepEndByNonEmpty p sep@ parses /one/ or more occurrences of @p@,
 -- separated and optionally ended by @sep@. Returns a non-empty list of values
@@ -177,7 +189,7 @@ endBy1 p sep = some (p <* sep)
 -- | @endByNonEmpty p sep@ parses /one/ or more occurrences of @p@, separated
 -- and ended by @sep@. Returns a non-empty list of values returned by @p@.
 endByNonEmpty :: Alternative m => m a -> m sep -> m (NonEmpty a)
-endByNonEmpty p sep = some1 (p <* sep)
+endByNonEmpty p sep = NonEmpty.some1 (p <* sep)
 {-# INLINE endByNonEmpty #-}
 
 -- | @endBy p sep@ parses /zero/ or more occurrences of @p@, separated
@@ -192,8 +204,12 @@ endBy p sep = many (p <* sep)
 -- equal to zero, the parser equals to @return []@. Returns a list of
 -- @n@ values returned by @p@.
 count :: Applicative m => Int -> m a -> m [a]
+#if MIN_VERSION_base(4,9,0)
+count = replicateM
+#else
 count n p | n <= 0    = pure []
           | otherwise = sequenceA (replicate n p)
+#endif
 {-# INLINE count #-}
 
 -- | @chainr p op x@ parses /zero/ or more occurrences of @p@,
@@ -273,7 +289,7 @@ class Alternative m => Parsing m where
   -- | A version of many that discards its input. Specialized because it
   -- can often be implemented more cheaply.
   skipMany :: m a -> m ()
-  skipMany p = () <$ many p
+  skipMany p = void (many p)
   {-# INLINE skipMany #-}
 
   -- | @skipSome p@ applies the parser @p@ /one/ or more times, skipping
@@ -419,6 +435,7 @@ instance (Parsing m, Monad m) => Parsing (IdentityT m) where
   notFollowedBy (IdentityT m) = IdentityT $ notFollowedBy m
   {-# INLINE notFollowedBy #-}
 
+#ifdef MIN_VERSION_parsec
 instance (Parsec.Stream s m t, Show t) => Parsing (Parsec.ParsecT s u m) where
   try           = Parsec.try
   (<?>)         = (Parsec.<?>)
@@ -427,7 +444,9 @@ instance (Parsec.Stream s m t, Show t) => Parsing (Parsec.ParsecT s u m) where
   unexpected    = Parsec.unexpected
   eof           = Parsec.eof
   notFollowedBy = Parsec.notFollowedBy
+#endif
 
+#ifdef MIN_VERSION_attoparsec
 instance Att.Chunk t => Parsing (Att.Parser t) where
   try             = Att.try
   (<?>)           = (Att.<?>)
@@ -436,6 +455,7 @@ instance Att.Chunk t => Parsing (Att.Parser t) where
   unexpected      = fail
   eof             = Att.endOfInput
   notFollowedBy p = optional p >>= maybe (pure ()) (unexpected . show)
+#endif
 
 #ifdef MIN_VERSION_binary
 instance Parsing B.Get where
